@@ -6,7 +6,15 @@ package "misc" or "util" seemed afraught with namespace collisions.
 """
 import os, sys, json
 
+version = '1.4.0'
+
 w3, private, public = None, None, None
+gasfactor = None
+
+def set_gasfactor(x):
+    global gasfactor
+    gasfactor = x
+    pass
 
 def set_private(x):
     global private
@@ -15,11 +23,10 @@ def set_private(x):
 
 def set_public(x):
     global public
+    set_default_address(x)
     public = x
     pass
 
-version = '1.3.0'
-
 def w3_connect(default_account):
     global w3
     from web3.auto import w3 as _w3
@@ -85,17 +92,21 @@ def old_deploy_contractAddress(name, *args):
     save_contractAddress(name, contractAddress)
     return contractAddress
 
-def deploy_contractAddress(name, *args):
+def deploy_contractAddress(name, *args, **kw):
     if private is None:
         return old_deploy_contractAddress(name, *args)
-    abi        = load_abi(name)
-    bytecode   = load_bytecode(name)
-    contract   = w3.eth.contract(abi=abi, bytecode=bytecode)
-    txn        = contract.constructor(*args).buildTransaction(
-        nonce = w3.eth.get_transaction_count(public))
-    signed_txn = w3.eth.account.sign_transaction(txn, private)
-    tx_hash = w3.eth.send_raw_transaction(signed_txn.rawTransaction)
-    tx_receipt = wait_for_tx(tx_hash)
+    abi         = load_abi(name)
+    bytecode    = load_bytecode(name)
+    contract    = w3.eth.contract(abi=abi, bytecode=bytecode)
+    if gasfactor is not None:
+        gas = contract.constructor(*args).estimateGas()
+        kw['gas']   = int(gasfactor * gas)
+        pass
+    kw['nonce'] = w3.eth.get_transaction_count(public)
+    txn         = contract.constructor(*args).buildTransaction(kw)
+    signed_txn  = w3.eth.account.sign_transaction(txn, private)
+    tx_hash     = w3.eth.send_raw_transaction(signed_txn.rawTransaction)
+    tx_receipt  = wait_for_tx(tx_hash)
     contractAddress = tx_receipt.contractAddress
     save_contractAddress(name, contractAddress)
     return contractAddress
@@ -107,7 +118,9 @@ def set_default_address(default_account):
     if type(default_account) == int:
         default_account = w3.eth.accounts[default_account]
         pass
+    print("ASDFSDF")
     w3.eth.default_account = default_account
+    print("ASDFSDF")
     return default_account
 
 class WrapMixin:
@@ -128,10 +141,16 @@ def old_wcall(contract, funcname, *args, _from=None, **kw):
 def wcall(contract, funcname, *args, _from=None, **kw):
     if private is None:
         return old_wcall(contract, funcname, *args, _from, **kw)
-    if _from: kw['from'] = _from
+    kw['from'] = _from if _from is not None else public
+    kw['from'] = public
     func = get_func(contract, funcname)
+    if gasfactor is not None:
+        gas = func(*args).estimateGas()
+        kw['gas']   = int(gasfactor * gas)
+        pass
     kw['nonce'] = w3.eth.get_transaction_count(public)
     txn = func(*args).buildTransaction(kw)
+    print("TXN", txn)
     signed_txn = w3.eth.account.sign_transaction(txn, private)
     tx_hash = w3.eth.send_raw_transaction(signed_txn.rawTransaction)
     tx_receipt = wait_for_tx(tx_hash)
